#!/usr/bin/env node
'use strict';

var path = require('path');
var qs = require('querystring');

var _ = require('lodash');
var simpleTimestamp = require('simple-timestamp');
var Elapsed = require('elapsed');
var antwar = require('antwar');
var chalk = require('chalk');
var program = require('commander');
var upperCaseFirst = require('upper-case-first');
var promiseFinally = require('promise-finally');

require('es6-promise').polyfill();

var lib = require('./lib');
var version = require('./package.json').version;

main();

function main() {
  var prettyConsole = {
    log: function log() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      console.log(simpleTimestamp(), chalk.green.apply(null, args));
    },
    info: function info() {
      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      console.info(simpleTimestamp(), chalk.blue.apply(null, args));
    },
    error: function error() {
      for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        args[_key3] = arguments[_key3];
      }

      console.error(simpleTimestamp(), chalk.bold.red.apply(null, args));
    },
    warn: function warn() {
      for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        args[_key4] = arguments[_key4];
      }

      console.warn(simpleTimestamp(), chalk.yellow.apply(null, args));
    }
  };
  var now = new Date();
  var defaultConfig = {
    webpackConfig: './webpack.config.js',
    blogRoot: 'blog',
    port: 3000,
    output: 'build',
    boilerplate: 'antwar-boilerplate',
    console: prettyConsole
  };

  program.version(version).option('-c, --config <file>', 'Path to configuration file ' + '(defaults to antwar.config.js) or `site` configuration as a querystring').option('-i, --init <directory>', 'Initialize a project').option('-I --install <theme>', 'Install a theme and attach it to project').option('-p, --plugin <directory>', 'Initialize a plugin').option('-b, --build', 'Build site').option('-l --list', 'List Antwar related packages').option('-s, --serve [port]', 'Serve site. Port (defaults to ' + defaultConfig.port + ')', parseInt).option('-d, --develop', 'Open a browser in development mode');

  program.parse(process.argv);

  // TODO: this would be a good place to validate configuration (push to core)
  // + show warnings about possible misspellings etc.
  var config = defaultConfig;

  // do not try to get antwar configuration when initializing a new project
  if (!program.init) {
    config = getConfig(defaultConfig, program.config);
  }

  config.port = parseInt(program.serve, 10) || config.port;
  config.output = program.init || program.plugin || config.output;

  // XXX: this can probably be merged somehow (map?)
  if (program.init) {
    execute(config.console, now, 'project initialization', lib.init, config, function () {
      config.console.info('Go to `' + config.output + '` and hit `npm start` to get started');
    });
  } else if (program.install) {
    config.theme = program.install;

    execute(config.console, now, 'installing', lib.install, config);
  } else if (program.plugin) {
    config.boilerplate = 'antwar-plugin-boilerplate';

    execute(config.console, now, 'plugin initialization', lib.init, config);
  } else if (program.build) {
    execute(config.console, now, 'building', antwar.build, config);
  } else if (program.serve) {
    execute(config.console, now, 'serving', lib.serve, config);
  } else if (program.list) {
    execute(config.console, now, 'listing', lib.list, config);
  } else if (program.deploy) {
    execute(config.console, now, 'deployment', lib.deploy, config);
  } else if (program.develop) {
    execute(config.console, now, 'developing', antwar.develop, config);
  } else if (!process.argv.slice(2).length) {
    program.outputHelp();
  }
}

function execute(console, startTime, name, command, config) {
  var doneCb = arguments.length <= 5 || arguments[5] === undefined ? noop : arguments[5];

  var upperCasedName = upperCaseFirst(name);

  console.log('Start ' + name + '\n');

  var p = command(config).then(function () {
    console.log('\n' + upperCasedName + ' finished');

    doneCb();
  }).catch(function (err) {
    console.error('\n' + upperCasedName + ' failed', err);
  });

  promiseFinally.default(p, function () {
    showElapsedTime(console, startTime);
  });
}

function noop() {}

function showElapsedTime(console, a, b) {
  var elapsedTime = new Elapsed(a, b || new Date());

  console.info('\nTime elapsed:', elapsedTime.optimal || elapsedTime.milliSeconds + ' ms');
}

function getConfig(defaultConfig, config) {
  try {
    var loadedConfig = {};

    // priority 1. qs 2. provided path 3. antwar.config.js
    // if nothing of these works, just use defaultConfig instead

    // assume it's a querystring if there's even one =
    // TODO: note that this doesn't parse nested properties (ie. foo.bar=34)
    // it might be nice to support that as well
    if (config && config.indexOf('=') > 0) {
      loadedConfig = qs.parse(config);
    } else {
      try {
        loadedConfig = require(path.join(process.cwd(), program.config || 'antwar.config.js'));
      } catch (e) {
        console.error(e);
      }
    }

    return _.merge(defaultConfig, loadedConfig);
  } catch (e) {
    console.error(e);
  }

  return defaultConfig;
}